package service

import (
	"context"
	"time"

	"git.pbiernat.dev/egommerce/api-entities/model"
	"git.pbiernat.dev/egommerce/go-api-pkg/fluentd"
	"github.com/georgysavva/scany/v2/pgxscan"
	"github.com/go-redis/redis/v8"
	"github.com/jackc/pgx/v5/pgxpool"
	"github.com/streadway/amqp"
)

type PriceService struct {
	dbConn *pgxpool.Pool
	redis  *redis.Client
	ebCh   *amqp.Channel
	log    *fluentd.Logger
}

func NewPriceService(dbConn *pgxpool.Pool, redis *redis.Client, chn *amqp.Channel, log *fluentd.Logger) *PriceService {
	return &PriceService{dbConn, redis, chn, log}
}

func (s *PriceService) FetchFromDB(ctx context.Context, prodID int) (float64, error) {
	product := new(model.ProductPriceModel)
	err := pgxscan.Get(ctx, s.dbConn, product, `SELECT id, pid, price FROM catalog.product WHERE id=$1`, prodID)
	if err != nil {
		return 0, err
	}

	return product.Price, nil // FIXME
}

func (s *PriceService) FetchFromCache(ctx context.Context, key string, itemType string) (any, error) {
	if itemType == "" {
		itemType = "string"
	}

	var (
		value any
		err   error
		cmd   = s.redis.Get(ctx, key)
	)
	switch itemType {
	case "string":
		value, err = cmd.Result()
	case "int":
		value, err = cmd.Int64()
	case "float":
		value, err = cmd.Float64()
	case "bool":
		value, err = cmd.Bool()
	}

	if err != nil {
		return 0, err
	}

	return value, nil
}

func (s *PriceService) SaveToCache(ctx context.Context, key string, value any, exp time.Duration) error {
	s.redis.Set(ctx, key, value, exp)

	return nil
}
